@@ -69,9 +69,24 @@ static const char *fast_reboot_disabled = NULL;
 
 void disable_fast_reboot(const char *reason)
 {
+	if (fast_reboot_disabled)
+		return;
+
+	prlog(PR_NOTICE, "RESET: Fast reboot disabled: %s\n", reason);
 	fast_reboot_disabled = reason;
 }
 
+void add_fast_reboot_dt_entries(void)
+{
+	dt_check_del_prop(opal_node, "fast-reboot");
+
+	if (fast_reboot_disabled) {
+		dt_add_property_string(opal_node, "fast-reboot", fast_reboot_disabled);
+	} else {
+		dt_add_property_string(opal_node, "fast-reboot", "okay");
+	}
+}
+
 /*
  * This is called by the reboot CPU after all other CPUs have been
  * quiesced and stopped, to perform various sanity checks on firmware
@@ -81,8 +96,12 @@ void disable_fast_reboot(const char *reason)
 static bool fast_reboot_sanity_check(void)
 {
 	if (!mem_check_all()) {
-		prlog(PR_NOTICE, "REST: Fast reboot failed due to inconsistent "
-				"firmware data\n");
+		disable_fast_reboot("Inconsistent firmware data");
+		return false;
+	}
+
+	if (!verify_romem()) {
+		disable_fast_reboot("Inconsistent firmware romem checksum");
 		return false;
 	}
 
@@ -110,13 +129,12 @@ void fast_reboot(void)
 	 * Ensure all other CPUs have left OPAL calls.
 	 */
 	if (!opal_quiesce(QUIESCE_HOLD, -1)) {
-		prlog(PR_NOTICE, "RESET: Fast reboot disabled because OPAL "
-				"quiesce timed out\n");
+		disable_fast_reboot("OPAL quiesce timeout");
 		return;
 	}
 
 	if (fast_reboot_disabled) {
-		prlog(PR_DEBUG, "RESET: Fast reboot disabled because %s\n",
+		prlog(PR_NOTICE, "RESET: Fast reboot disabled: %s\n",
 		      fast_reboot_disabled);
 		opal_quiesce(QUIESCE_RESUME, -1);
 		return;
@@ -363,9 +381,7 @@ void __noreturn fast_reboot_entry(void)
 		 * preload.
 		 *
 		 * Mambo may have embedded payload here, so don't clear
-		 * it at all. The OS may have already overwritten it, so
-		 * mambo really should reserve memory regions for this, if
-		 * fast reboot is to work reliably.
+		 * it at all.
 		 */
 		memset(KERNEL_LOAD_BASE, 0, KERNEL_LOAD_SIZE);
 		memset(INITRAMFS_LOAD_BASE, 0, INITRAMFS_LOAD_SIZE);
@@ -374,6 +390,9 @@ void __noreturn fast_reboot_entry(void)
 	/* Start preloading kernel and ramdisk */
 	start_preload_kernel();
 
+	/* Start clearing memory */
+	start_mem_region_clear_unused();
+
 	/* Poke the consoles (see comments in the code there) */
 	fsp_console_reset();
 
@@ -399,7 +418,7 @@ void __noreturn fast_reboot_entry(void)
 
 	ipmi_set_fw_progress_sensor(IPMI_FW_PCI_INIT);
 
-	mem_region_clear_unused();
+	wait_mem_region_clear_unused();
 
 	/* Load and boot payload */
 	load_and_boot_kernel(true);
