@@ -29,27 +29,6 @@
 static struct phb *phbs[MAX_PHB_ID];
 int last_phb_id = 0;
 
-#define PCITRACE(_p, _bdfn, fmt, a...) \
-	prlog(PR_TRACE, "PHB#%04x:%02x:%02x.%x " fmt,	\
-	      (_p)->opal_id,				\
-	      ((_bdfn) >> 8) & 0xff,			\
-	      ((_bdfn) >> 3) & 0x1f, (_bdfn) & 0x7, ## a)
-#define PCIDBG(_p, _bdfn, fmt, a...) \
-	prlog(PR_DEBUG, "PHB#%04x:%02x:%02x.%x " fmt,	\
-	      (_p)->opal_id,				\
-	      ((_bdfn) >> 8) & 0xff,			\
-	      ((_bdfn) >> 3) & 0x1f, (_bdfn) & 0x7, ## a)
-#define PCINOTICE(_p, _bdfn, fmt, a...) \
-	prlog(PR_NOTICE, "PHB#%04x:%02x:%02x.%x " fmt,	\
-	      (_p)->opal_id,				\
-	      ((_bdfn) >> 8) & 0xff,			\
-	      ((_bdfn) >> 3) & 0x1f, (_bdfn) & 0x7, ## a)
-#define PCIERR(_p, _bdfn, fmt, a...) \
-	prlog(PR_ERR, "PHB#%04x:%02x:%02x.%x " fmt,	\
-	      (_p)->opal_id,				\
-	      ((_bdfn) >> 8) & 0xff,			\
-	      ((_bdfn) >> 3) & 0x1f, (_bdfn) & 0x7, ## a)
-
 /*
  * Generic PCI utilities
  */
@@ -401,8 +380,10 @@ static void pci_slot_set_power_state(struct phb *phb,
 
 	if (state == PCI_SLOT_POWER_OFF) {
 		/* Bail if there're something connected */
-		if (!list_empty(&pd->children))
+		if (!list_empty(&pd->children)) {
+			PCIERR(phb, pd->bdfn, "Attempted to power off slot with attached devices!\n");
 			return;
+		}
 
 		pci_slot_add_flags(slot, PCI_SLOT_FLAG_BOOTUP);
 		rc = slot->ops.get_power_state(slot, &cur_state);
@@ -915,7 +896,9 @@ uint8_t pci_scan_bus(struct phb *phb, uint8_t bus, uint8_t max_bus,
 		pci_cfg_write8(phb, pd->bdfn, PCI_CFG_SUBORDINATE_BUS, max_sub);
 		next_bus = max_sub + 1;
 
-		pci_slot_set_power_state(phb, pd, PCI_SLOT_POWER_OFF);
+		/* power off the slot if there's nothing below it */
+		if (list_empty(&pd->children))
+			pci_slot_set_power_state(phb, pd, PCI_SLOT_POWER_OFF);
 	}
 
 	return max_sub;
@@ -1748,7 +1731,7 @@ static void __pci_init_slots(void)
 	if (platform.pci_probe_complete)
 		platform.pci_probe_complete();
 
-	prlog(PR_DEBUG, "PCI Summary:\n");
+	prlog(PR_NOTICE, "PCI Summary:\n");
 
 	for (i = 0; i < ARRAY_SIZE(phbs); i++) {
 		if (!phbs[i])
