@@ -36,8 +36,6 @@ void npu2_scom_write(uint64_t gcid, uint64_t scom_base,
 #define	MCD_BANK_CN_VALID	PPC_BIT(0)
 #define	MCD_BANK_CN_SIZE	PPC_BITMASK(13,29)
 #define	MCD_BANK_CN_ADDR	PPC_BITMASK(33,63)
-#define PB_CENT_HP_MODE_CURR	0x5011c0c
-#define  PB_CFG_CHG_RATE_GP_MASTER PPC_BIT(2)
 #define PB_CENT_MODE		0x5011c0a
 #define  PB_CFG_CHIP_ADDR_EXTENSION_MASK_CENT PPC_BITMASK(42,48)
 
@@ -46,16 +44,20 @@ void npu2_scom_write(uint64_t gcid, uint64_t scom_base,
 	(((stack) << 20) | ((block) << 16) | (offset))
 
 #define NPU2_NTL_REG_OFFSET(ndev, offset)				\
-	NPU2_REG_OFFSET(NPU2_STACK_STCK_0 + ((ndev)->index >> 1),       \
-			NPU2_BLOCK_NTL0 + ((ndev)->index % 2)*2, offset)
+	NPU2_REG_OFFSET(NPU2_STACK_STCK_0 + ((ndev)->brick_index >> 1),       \
+			NPU2_BLOCK_NTL0 + ((ndev)->brick_index % 2)*2, offset)
 
 #define NPU2_NTLU_REG_OFFSET(ndev, offset)				\
-	NPU2_REG_OFFSET(NPU2_STACK_STCK_0U + ((ndev)->index >> 1),       \
-			NPU2_BLOCK_NTL0 + ((ndev)->index % 2)*2, offset)
+	NPU2_REG_OFFSET(NPU2_STACK_STCK_0U + ((ndev)->brick_index >> 1),       \
+			NPU2_BLOCK_NTL0 + ((ndev)->brick_index % 2)*2, offset)
 
 #define NPU2_DL_REG_OFFSET(ndev, offset)	\
-	NPU2_REG_OFFSET(((ndev)->index >> 1),				\
-			8 + ((ndev)->index % 2)*2, offset)
+	NPU2_REG_OFFSET(((ndev)->brick_index >> 1),				\
+			8 + ((ndev)->brick_index % 2)*2, offset)
+
+#define NPU2_SM_REG_OFFSET(ndev, sm, offset)				\
+	NPU2_REG_OFFSET(NPU2_STACK_STCK_0 + ((ndev)->brick_index >> 1),	\
+			NPU2_BLOCK_SM_0 + (sm), offset)
 
 /* Get the offset for this register */
 #define NPU2_REG(reg) \
@@ -150,9 +152,26 @@ void npu2_scom_write(uint64_t gcid, uint64_t scom_base,
 #define NPU2_LOW_WATER_MARKS			0x040
 #define   NPU2_LOW_WATER_MARKS_ENABLE_MACHINE_ALLOC PPC_BIT(51)
 #define NPU2_HIGH_WATER_MARKS			0x048
-#define NPU2_RELAXED_ORDERING_CFG0		0x050
-#define NPU2_RELAXED_ORDERING_CFG1		0x058
-#define NPU2_RELAXED_ORDERING_CFG2		0x060
+#define NPU2_RELAXED_ORDERING_CFG(n)		(0x050 + (n)*8)
+#define   NPU2_RELAXED_ORDERING_SOURCE(n)	(PPC_BITMASK(0,31) >> ((n)*32))
+#define     NPU2_RELAXED_ORDERING_SOURCE_ENA	PPC_BITMASK32(0,3)
+#define     NPU2_RELAXED_ORDERING_SOURCE_WRENA	PPC_BIT32(0)
+#define     NPU2_RELAXED_ORDERING_SOURCE_RDENA	PPC_BIT32(1)
+#define     NPU2_RELAXED_ORDERING_SOURCE_AWENA	PPC_BIT32(2)
+#define     NPU2_RELAXED_ORDERING_SOURCE_ARENA	PPC_BIT32(3)
+#define     NPU2_RELAXED_ORDERING_SOURCE_PECSEL	PPC_BITMASK32(4,5)
+#define     NPU2_RELAXED_ORDERING_SOURCE_GRPCHP	PPC_BITMASK32(6,9)
+#define     NPU2_RELAXED_ORDERING_SOURCE_WRMIN	PPC_BITMASK32(10,14)
+#define     NPU2_RELAXED_ORDERING_SOURCE_WRMAX	PPC_BITMASK32(15,19)
+#define     NPU2_RELAXED_ORDERING_SOURCE_RDMIN	PPC_BITMASK32(20,25)
+#define     NPU2_RELAXED_ORDERING_SOURCE_RDMAX	PPC_BITMASK32(26,31)
+#define   NPU2_RELAXED_ORDERING_CMD_CL_DMA_W	PPC_BIT(0)
+#define   NPU2_RELAXED_ORDERING_CMD_CL_DMA_W_HP	PPC_BIT(1)
+#define   NPU2_RELAXED_ORDERING_CMD_CL_DMA_INJ	PPC_BIT(2)
+#define   NPU2_RELAXED_ORDERING_CMD_PR_DMA_INJ	PPC_BIT(3)
+#define   NPU2_RELAXED_ORDERING_CMD_DMA_PR_W	PPC_BIT(4)
+#define   NPU2_RELAXED_ORDERING_CMD_CL_RD_NC_F0	PPC_BIT(5)
+#define   NPU2_RELAXED_ORDERING_SOURCE4_RDENA	PPC_BIT(29)
 #define NPU2_NTL0_BAR				0x068
 #define NPU2_NTL1_BAR				0x070
 #define   NPU2_NTL_BAR_ENABLE			PPC_BIT(0)
@@ -268,6 +287,11 @@ void npu2_scom_write(uint64_t gcid, uint64_t scom_base,
 
 /* NTL block registers */
 #define NPU2_NTL_MISC_CFG2(ndev)		NPU2_NTL_REG_OFFSET(ndev, 0x000)
+#define   NPU2_NTL_MISC_CFG2_BRICK_ENABLE	PPC_BIT(0)
+#define   NPU2_NTL_MISC_CFG2_NDL_RX_PARITY_ENA	PPC_BIT(16)
+#define   NPU2_NTL_MISC_CFG2_NDL_TX_PARITY_ENA	PPC_BIT(17)
+#define   NPU2_NTL_MISC_CFG2_NDL_PRI_PARITY_ENA	PPC_BIT(18)
+#define   NPU2_NTL_MISC_CFG2_RCV_CREDIT_OVERFLOW_ENA PPC_BIT(19)
 #define NPU2_NTL_MISC_CFG3(ndev)		NPU2_NTL_REG_OFFSET(ndev, 0x008)
 #define NPU2_NTL_ERR_HOLD1(ndev)		NPU2_NTL_REG_OFFSET(ndev, 0x010)
 #define NPU2_NTL_ERR_MASK1(ndev)		NPU2_NTL_REG_OFFSET(ndev, 0x018)
@@ -555,6 +579,8 @@ void npu2_scom_write(uint64_t gcid, uint64_t scom_base,
 #define NPU2_XTS_MMIO_ATSD_LAUNCH		0x000
 #define NPU2_XTS_MMIO_ATSD_AVA			0x008
 #define NPU2_XTS_MMIO_ATSD_STATUS		0x010
+#define MMIO_ATSD_ADDR(p, n) 			(u64) p + NPU2_REG_OFFSET(NPU2_STACK_ATSD,\
+						NPU2_BLOCK_ATSD##n, NPU2_XTS_MMIO_ATSD_LAUNCH)
 
 /* ALTD SCOM addresses */
 #define NPU2_MISC_SCOM_IND_SCOM_ADDR		0x68e
@@ -715,6 +741,8 @@ void npu2_scom_write(uint64_t gcid, uint64_t scom_base,
 #define OB3_ODL0_STATUS				0xC01082C
 #define OB3_ODL1_STATUS				0xC01082D
 #define   OB_ODL_STATUS_TRAINED_MODE		PPC_BITMASK(0,3)
+#define   OB_ODL_STATUS_RX_TRAINED_LANES	PPC_BITMASK(16, 23)
+#define   OB_ODL_STATUS_TX_TRAINED_LANES	PPC_BITMASK(24, 31)
 #define   OB_ODL_STATUS_TRAINING_STATE_MACHINE	PPC_BITMASK(49, 51)
 
 #define OB0_ODL0_TRAINING_STATUS		0x901082E
@@ -723,4 +751,20 @@ void npu2_scom_write(uint64_t gcid, uint64_t scom_base,
 #define OB3_ODL1_TRAINING_STATUS		0xC01082F
 #define   OB_ODL_TRAINING_STATUS_STS_RX_PATTERN_B PPC_BITMASK(8, 15)
 
+#define OB0_ODL0_ENDPOINT_INFO			0x9010832
+#define OB0_ODL1_ENDPOINT_INFO			0x9010833
+#define OB3_ODL0_ENDPOINT_INFO			0xC010832
+#define OB3_ODL1_ENDPOINT_INFO			0xC010833
+
+/* Registers and bits used to clear the L2 and L3 cache */
+#define L2_PRD_PURGE_CMD_REG 			0x1080E
+#define L2_PRD_PURGE_CMD_REG_BUSY 		0x0040000000000000
+#define L2_PRD_PURGE_CMD_TYPE_MASK		PPC_BIT(1) | PPC_BIT(2) | PPC_BIT(3) | PPC_BIT(4)
+#define L2_PRD_PURGE_CMD_TRIGGER		PPC_BIT(0)
+#define L2CAC_FLUSH				0x0
+#define L3_PRD_PURGE_REG			0x1180E
+#define L3_PRD_PURGE_REQ			PPC_BIT(0)
+#define L3_PRD_PURGE_TTYPE_MASK 		PPC_BIT(1) | PPC_BIT(2) | PPC_BIT(3) | PPC_BIT(4)
+#define L3_FULL_PURGE				0x0
+
 #endif /* __NPU2_REGS_H */
