@@ -23,6 +23,27 @@
 #include <bitmap.h>
 #include <ccan/list/list.h>
 
+#define PCITRACE(_p, _bdfn, fmt, a...) \
+	prlog(PR_TRACE, "PHB#%04x:%02x:%02x.%x " fmt,	\
+	      (_p)->opal_id,				\
+	      ((_bdfn) >> 8) & 0xff,			\
+	      ((_bdfn) >> 3) & 0x1f, (_bdfn) & 0x7, ## a)
+#define PCIDBG(_p, _bdfn, fmt, a...) \
+	prlog(PR_DEBUG, "PHB#%04x:%02x:%02x.%x " fmt,	\
+	      (_p)->opal_id,				\
+	      ((_bdfn) >> 8) & 0xff,			\
+	      ((_bdfn) >> 3) & 0x1f, (_bdfn) & 0x7, ## a)
+#define PCINOTICE(_p, _bdfn, fmt, a...) \
+	prlog(PR_NOTICE, "PHB#%04x:%02x:%02x.%x " fmt,	\
+	      (_p)->opal_id,				\
+	      ((_bdfn) >> 8) & 0xff,			\
+	      ((_bdfn) >> 3) & 0x1f, (_bdfn) & 0x7, ## a)
+#define PCIERR(_p, _bdfn, fmt, a...) \
+	prlog(PR_ERR, "PHB#%04x:%02x:%02x.%x " fmt,	\
+	      (_p)->opal_id,				\
+	      ((_bdfn) >> 8) & 0xff,			\
+	      ((_bdfn) >> 3) & 0x1f, (_bdfn) & 0x7, ## a)
+
 struct pci_device;
 struct pci_cfg_reg_filter;
 
@@ -90,6 +111,14 @@ struct pci_device {
 	uint32_t		pcrf_end;
 	struct list_head	pcrf;
 
+	/*
+	 * Relaxed ordering is a feature which allows PCIe devices accessing GPU
+	 * memory to bypass the normal PCIe ordering rules to increase
+	 * performance. It is enabled on a per-PEC basis so every device on a
+	 * PEC must support it before we can enable it.
+	 */
+	bool                    allow_relaxed_ordering;
+
 	struct dt_node		*dn;
 	struct pci_slot		*slot;
 	struct pci_device	*parent;
